import { Logger } from '@terascope/utils';
import * as i from '../../interfaces';
export declare class PromMetrics {
    readonly metricList: i.MetricList;
    default_labels: Record<string, string>;
    prefix: string;
    private metricExporter;
    context: i.FoundationContext;
    apiConfig: i.PromMetricsAPIConfig;
    apiRunning: boolean;
    logger: Logger;
    constructor(context: i.FoundationContext, logger: Logger);
    /**
     * Initialize the promMetrics API and create an exporter server. Will not initialize a new API
     * if in native clustering or if the API has already been initialized.
     * @param {i.PromMetricsInitConfig} config PromMetricsInitConfig values override those
     * of terafoundation. This allows jobs to set different metrics configurations than the master.
     * @returns {Promise<boolean>} Was the API initialized
     */
    init(config: i.PromMetricsInitConfig): Promise<boolean>;
    /**
     * [set metric to value]
     * @param  {string} name [metric name]
     * @param  {Record<string, string>} labels [list of labels and labels values]
     * @param  {number} value [metric value]
     * @return {void}
     */
    set(name: string, labels: Record<string, string>, value: number): void;
    /**
     * [increment metric by value]
     * @param  {string} name [metric name]
     * @param  {Record<string, string>} labels [list of labels and labels values]
     * @param  {number} value [metric value, default = 1]
     * @return {void}
     */
    inc(name: string, labels: Record<string, string>, value?: number): void;
    /**
     * [decrement metric by value]
     * @param  {string} name [metric name]
     * @param  {Record<string, string>} labels [list of labels and labels values]
     * @param  {number} value [metric value, default = 1]
     * @return {void}
     */
    dec(name: string, labels: Record<string, string>, value?: number): void;
    /**
     * [observe value, used by summary metric type]
     * @param  {string} name [metric name]
     * @param  {Array<string, string>} labels [list of labels and labels values]
     * @param  {number} value [metric value]
     * @return {void}
     */
    observe(name: string, labels: Record<string, string>, value: number): void;
    /**
     * [addMetric (define) new metric]
     * @param  {string} name [metric name]
     * @param  {string} help [metric help]
     * @param  {Array<string>} labelsNames [list of label names]
     * @param  {'gauge' | 'counter' | 'histogram'} type [gauge,counter,histogram,or summary]
     * @param  {Array<number>} buckets [default buckets]
     * @return {Promise<void>}
     */
    addMetric(name: string, help: string, labelsNames: Array<string>, type: 'gauge' | 'counter' | 'histogram', buckets?: Array<number>): Promise<void>;
    /**
     * [hasMetric check if metricList contains metric]
     * @param  {string} name [metric name]
     * @return {boolean}
     */
    hasMetric(name: string): boolean;
    /**
     * [deleteMetric delete metric from metricList]
     * @param  {string} name [metric name]
     * @return {Promise<boolean>} [was the metric successfully deleted]
     */
    deleteMetric(name: string): Promise<boolean>;
    /**
     * [addSummary (define) new summary metric]
     * @param  {string} name [metric name]
     * @param  {string} help [metric help]
     * @param  {Array<string>} labelsNames [list of label names]
     * @param  {Array<number>} percentiles [metric percentiles, default[0.01, 0.1, 0.9, 0.99] ]
     * @param  {number} maxAgeSeconds [how old a bucket can be before it is reset ]
     * @param  {number} ageBuckets [how many buckets for sliding window ]
     * @return {Promise<void>}
     */
    addSummary(name: string, help: string, labelsNames: Array<string>, maxAgeSeconds?: number, ageBuckets?: number, percentiles?: Array<number>): Promise<void>;
    private _createGaugeMetric;
    private _createCounterMetric;
    private _createHistogramMetric;
    private _createSummaryMetric;
    private _setPodName;
    private createAPI;
    verifyAPI(): boolean;
    shutdown(): Promise<void>;
}
//# sourceMappingURL=prom-metrics-api.d.ts.map