"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PromMetrics = void 0;
const os_1 = __importDefault(require("os"));
const prom_client_1 = require("prom-client");
const exporter_1 = __importDefault(require("./exporter"));
class PromMetrics {
    constructor(context, logger) {
        this.context = context;
        this.apiRunning = false;
        this.apiConfig = {};
        this.prefix = '';
        this.logger = logger.child({ module: 'prom_metrics' });
        this.metricList = {};
    }
    /**
     * Initialize the promMetrics API and create an exporter server. Will not initialize a new API
     * if in native clustering or if the API has already been initialized.
     * @param {i.PromMetricsInitConfig} config PromMetricsInitConfig values override those
     * of terafoundation. This allows jobs to set different metrics configurations than the master.
     * @returns {Promise<boolean>} Was the API initialized
     */
    async init(config) {
        const { terafoundation, teraslice } = config.context.sysconfig;
        const metricsEnabledInTF = terafoundation.prom_metrics_enabled;
        const portToUse = config.port || terafoundation.prom_metrics_port;
        if (teraslice.cluster_manager_type === 'native') {
            this.logger.warn('Skipping PromMetricsAPI initialization: incompatible with native clustering.');
            return false;
        }
        // If prom_metrics_add_default is defined in jobSpec use that value.
        // If not use the terafoundation value.
        const useDefaultMetrics = config.default_metrics !== undefined
            ? config.default_metrics
            : terafoundation.prom_metrics_add_default;
        // If prom_metrics_enabled is true in jobConfig, or if not specified in
        // jobSpec and true in terafoundation, then we enable metrics.
        if (config.metrics_enabled_by_job === true
            || (config.metrics_enabled_by_job === undefined && metricsEnabledInTF)) {
            const apiConfig = {
                assignment: config.assignment,
                port: portToUse,
                default_metrics: useDefaultMetrics,
                labels: config.labels,
                prefix: config.prefix
            };
            this.prefix = apiConfig.prefix || `teraslice_${apiConfig.assignment}_`;
            this.default_labels = {
                name: this.context.sysconfig.teraslice.name,
                assignment: apiConfig.assignment,
                ...apiConfig.labels
            };
            await this.createAPI(apiConfig);
            return true;
        }
        this.logger.warn('Cannot create PromMetricsAPI because metrics are disabled.');
        return false;
    }
    /**
     * [set metric to value]
     * @param  {string} name [metric name]
     * @param  {Record<string, string>} labels [list of labels and labels values]
     * @param  {number} value [metric value]
     * @return {void}
     */
    set(name, labels, value) {
        const metric = this.metricList[name];
        if (!metric.functions || !metric.metric) {
            throw new Error(`Metric ${name} is not setup`);
        }
        if (metric.metric instanceof prom_client_1.Gauge) {
            const labelValues = Object.keys(labels).map((key) => labels[key]);
            const res = metric.metric.labels(...labelValues.concat(Object.values(this.default_labels)));
            res.set(value);
        }
        else {
            throw new Error(`set not available on ${name} metric`);
        }
    }
    /**
     * [increment metric by value]
     * @param  {string} name [metric name]
     * @param  {Record<string, string>} labels [list of labels and labels values]
     * @param  {number} value [metric value, default = 1]
     * @return {void}
     */
    inc(name, labels, value = 1) {
        const metric = this.metricList[name];
        if (!metric.functions || !metric.metric) {
            throw new Error(`Metric ${name} is not setup`);
        }
        if (metric.metric instanceof prom_client_1.Counter || metric.metric instanceof prom_client_1.Gauge) {
            const labelValues = Object.keys(labels).map((key) => labels[key]);
            const res = metric.metric.labels(...labelValues.concat(Object.values(this.default_labels)));
            res.inc(value);
        }
        else {
            throw new Error(`inc not available on ${name} metric`);
        }
    }
    /**
     * [decrement metric by value]
     * @param  {string} name [metric name]
     * @param  {Record<string, string>} labels [list of labels and labels values]
     * @param  {number} value [metric value, default = 1]
     * @return {void}
     */
    dec(name, labels, value = 1) {
        const metric = this.metricList[name];
        if (!metric.functions || !metric.metric) {
            throw new Error(`Metric ${name} is not setup`);
        }
        if (metric.metric instanceof prom_client_1.Gauge) {
            const labelValues = Object.keys(labels).map((key) => labels[key]);
            const res = metric.metric.labels(...labelValues.concat(Object.values(this.default_labels)));
            res.dec(value);
        }
        else {
            throw new Error(`dec not available on ${name} metric`);
        }
    }
    /**
     * [observe value, used by summary metric type]
     * @param  {string} name [metric name]
     * @param  {Array<string, string>} labels [list of labels and labels values]
     * @param  {number} value [metric value]
     * @return {void}
     */
    observe(name, labels, value) {
        const metric = this.metricList[name];
        if (!metric.functions || !metric.metric) {
            throw new Error(`Metric ${name} is not setup`);
        }
        if (metric.metric instanceof prom_client_1.Summary || metric.metric instanceof prom_client_1.Histogram) {
            const labelValues = Object.keys(labels).map((key) => labels[key]);
            const res = metric.metric.labels(...labelValues.concat(Object.values(this.default_labels)));
            res.observe(value);
        }
        else {
            throw new Error(`observe not available on ${name} metric`);
        }
    }
    /**
     * [addMetric (define) new metric]
     * @param  {string} name [metric name]
     * @param  {string} help [metric help]
     * @param  {Array<string>} labelsNames [list of label names]
     * @param  {'gauge' | 'counter' | 'histogram'} type [gauge,counter,histogram,or summary]
     * @param  {Array<number>} buckets [default buckets]
     * @return {Promise<void>}
     */
    async addMetric(name, help, labelsNames, type, buckets = [0.1, 5, 15, 50, 100, 500]) {
        if (!this.hasMetric(name)) {
            const fullname = this.prefix + name;
            if (type === 'gauge') {
                this.metricList[name] = this._createGaugeMetric(fullname, help, labelsNames.concat(Object.keys(this.default_labels)));
            }
            if (type === 'counter') {
                this.metricList[name] = this._createCounterMetric(fullname, help, labelsNames.concat(Object.keys(this.default_labels)));
            }
            if (type === 'histogram') {
                this.metricList[name] = this._createHistogramMetric(fullname, help, labelsNames.concat(Object.keys(this.default_labels)), buckets);
            }
        }
        else {
            this.logger.info(`metric ${name} already defined in metric list`);
        }
    }
    /**
     * [hasMetric check if metricList contains metric]
     * @param  {string} name [metric name]
     * @return {boolean}
     */
    hasMetric(name) {
        return (name in this.metricList);
    }
    /**
     * [deleteMetric delete metric from metricList]
     * @param  {string} name [metric name]
     * @return {Promise<boolean>} [was the metric successfully deleted]
     */
    async deleteMetric(name) {
        let deleted = false;
        const fullname = this.prefix + name;
        this.logger.info(`delete metric ${fullname}`);
        if (this.hasMetric(name)) {
            deleted = delete this.metricList[name];
            try {
                await this.metricExporter.deleteMetric(fullname);
            }
            catch (err) {
                deleted = false;
                throw new Error(`unable to delete metric ${fullname} from exporter`);
            }
        }
        else {
            throw new Error(`metric ${name} not defined in metric list`);
        }
        return deleted;
    }
    /**
     * [addSummary (define) new summary metric]
     * @param  {string} name [metric name]
     * @param  {string} help [metric help]
     * @param  {Array<string>} labelsNames [list of label names]
     * @param  {Array<number>} percentiles [metric percentiles, default[0.01, 0.1, 0.9, 0.99] ]
     * @param  {number} maxAgeSeconds [how old a bucket can be before it is reset ]
     * @param  {number} ageBuckets [how many buckets for sliding window ]
     * @return {Promise<void>}
     */
    async addSummary(name, help, labelsNames, maxAgeSeconds = 600, ageBuckets = 5, percentiles = [0.01, 0.1, 0.9, 0.99]) {
        if (!(name in this.metricList)) {
            const fullname = this.prefix + name;
            this.metricList[name] = this._createSummaryMetric(fullname, help, labelsNames.concat(Object.keys(this.default_labels)), percentiles, maxAgeSeconds, ageBuckets);
        }
    }
    _createGaugeMetric(name, help, labelsNames) {
        const gauge = new prom_client_1.Gauge({
            name,
            help,
            labelNames: labelsNames,
        });
        return { name, metric: gauge, functions: new Set(['inc', 'dec', 'set']) };
    }
    _createCounterMetric(name, help, labelsNames) {
        const counter = new prom_client_1.Counter({
            name,
            help,
            labelNames: labelsNames,
        });
        return { name, metric: counter, functions: new Set(['inc']) };
    }
    _createHistogramMetric(name, help, labelsNames, buckets) {
        const histogram = new prom_client_1.Histogram({
            name,
            help,
            labelNames: labelsNames,
            buckets
        });
        return { name, metric: histogram, functions: new Set(['observe']) };
    }
    _createSummaryMetric(name, help, labelsNames, percentiles, ageBuckets, maxAgeSeconds) {
        const histogram = new prom_client_1.Summary({
            name,
            help,
            labelNames: labelsNames,
            percentiles,
            maxAgeSeconds,
            ageBuckets
        });
        return { name, metric: histogram, functions: new Set(['observe']) };
    }
    _setPodName() {
        // in a pod the hostname is the pod name
        const host = os_1.default.hostname();
        if (host.startsWith('ts-wkr-')) {
            this.default_labels.pod_name = host;
        }
    }
    async createAPI(apiConfig) {
        this._setPodName();
        try {
            if (!this.metricExporter) {
                apiConfig.labels = Object.assign({}, apiConfig.labels, this.default_labels);
                this.metricExporter = new exporter_1.default(this.logger);
                await this.metricExporter.create(apiConfig);
                this.logger.info(`prom_metrics_API exporter created on port ${apiConfig.port}`);
            }
        }
        catch (err) {
            this.logger.info('prom_metrics_API exporter already running');
            this.logger.error(err);
        }
        this.apiRunning = true;
    }
    verifyAPI() {
        return this.apiRunning;
    }
    async shutdown() {
        this.logger.info('prom_metrics_API exporter shutdown');
        try {
            await this.metricExporter.shutdown();
            this.apiRunning = false;
        }
        catch (err) {
            this.logger.error(`shutting down metric exporter ${err}`);
        }
    }
}
exports.PromMetrics = PromMetrics;
//# sourceMappingURL=prom-metrics-api.js.map